// Adapted for LHAPDFv4 by Mike Whalley.
// Adapted for LHAPDFv5 by Craig Group & Mike Whalley.
// Fortran portability and string passing by Andy Buckley.

#include "LHAPDF/LHAPDF.h"
using namespace std;

namespace LHAPDF {

  string _getFileExtension(SetType type) {
    switch(type) {
    case LHPDF:
      return "LHpdf";
    case LHGRID:
      return "LHgrid";
    }
  }


  void initPDFByName(const string& name, SetType type, int memset) {
    initPDFSetByName(name, type);
    initPDF(memset);
  }

  void initPDFByName(const string& filename, int memset) {
    initPDFSetByName(filename);
    initPDF(memset);
  }

  void initPDFSet(const string& path) { 
    char cpath[1000];
    strcpy(cpath, path.c_str());
    finitpdfset_(cpath, path.length());
  }

  void initPDFSetByName(const string& name, SetType type) {
    initPDFSetByName(name + "." + _getFileExtension(type));
  }
  
  void initPDFSetByName(const string& filename) { 
    char cfilename[1000];
    strcpy(cfilename, filename.c_str());
    finitpdfsetbyname_(cfilename, filename.length());
  }

  void initPDF(int memset) { 
    finitpdf_(&memset); 
  }

  void getDescription() {
    fgetdesc_();
  }


  void initPDFByNameM(int nset, const string& name, SetType type, int memset) {
    initPDFSetByNameM(nset, name, type);
    initPDFM(nset, memset);
  }

  void initPDFByNameM(int nset, const string& filename, int memset) {
    initPDFSetByNameM(nset, filename);
    initPDFM(nset, memset);
  }

  void initPDFSetM(int nset, const string& path) {
    char cpath[1000];
    strcpy(cpath, path.c_str());
    finitpdfsetm_(&nset, cpath, path.length());
  }

  void initPDFSetByNameM(int nset, const string& name, SetType type) {
    initPDFSetByNameM(nset, name + "." + _getFileExtension(type));
  }

  void initPDFSetByNameM(int nset, const string& filename) {
    char cfilename[1000];
    strcpy(cfilename, filename.c_str());
    finitpdfsetbynamem_(&nset, cfilename, filename.length());
  }

  void initPDFM(int nset, int memset) {
    finitpdfm_(&nset, &memset);
  }

  void getDescriptionM(int nset) {
    fgetdescm_(&nset);
  }




  vector<double> xfx(double x, double Q) {  
    vector<double> r(13);
    fevolvepdf_(&x, &Q, &r[0]);
    return r;
  }

  void xfx(double x, double Q, double* results) {
    fevolvepdf_(&x, &Q, results);
  }

  double xfx(double x, double Q, int fl) {  
    vector<double> r(13);
    fevolvepdf_(&x, &Q, &r[0]);
    return r[fl+6];
  }

  vector<double> xfxp(double x, double Q, double P2, int ip) {  
    vector<double> r(13);
    fevolvepdfp_(&x, &Q, &P2, &ip, &r[0]);
    return r;
  }

  void xfxp(double x, double Q, double P2, int ip, double* results) {
    fevolvepdfp_(&x, &Q, &P2, &ip, results);
  }

  double xfxp(double x, double Q, double P2, int ip, int fl) {  
    vector<double> r(13);
    fevolvepdfp_(&x, &Q, &P2, &ip, &r[0]);
    return r[fl+6];
  }

  vector<double> xfxa(double x, double Q, double a) {  
    vector<double> r(13);
    fevolvepdfa_(&x, &Q, &a, &r[0]);
    return r;
  }

  void xfxa(double x, double Q, double a, double* results) {
    fevolvepdfa_(&x, &Q, &a, results);
  }

  double xfxa(double x, double Q, double a, int fl) {  
    vector<double> r(13);
    fevolvepdfa_(&x, &Q, &a, &r[0]);
    return r[fl+6];
  }

  vector<double> xfxphoton(double x, double Q) {  
    vector<double> r(13);  
    double mphoton;
    fevolvepdfphoton_(&x, &Q, &r[0], &mphoton );
    r.push_back(mphoton);
    return r;
  }

  void xfxphoton(double x, double Q, double* results) {
    fevolvepdfphoton_(&x, &Q, results, results+13);
  }

  double xfxphoton(double x, double Q, int fl) {  
    vector<double> r(13);
    double mphoton;
    fevolvepdfphoton_(&x, &Q, &r[0], &mphoton);
    if (fl == 7) return mphoton;
    return r[fl+6];
  }

  int numberPDF() {
    int N;
    fnumberpdf_(&N); 
    return N;
  }

  double alphasPDF(double Q) {
    double a;
    falphaspdf_(&Q, &a);
    return a;
  }

  int getOrderPDF() {
    int N;
    fgetorderpdf_(&N);
    return N;
  }

  int getOrderAlphaS() {
    int N;
    fgetorderas_(&N);
    return N;
  }

  double getQMass(int f) {
    double m;
    fgetqmass_(&f, &m);
    return m;
  }

  double getThreshold(int f) {
    double m;
    fgetthreshold_(&f, &m);
    return m;
  }

  int getNf() {
    int N; 
    fgetnf_(&N); 
    return N;
  }

  double getLam4(int m) {
    double l;
    fgetlam4_(&m, &l);
    return l;
  }

  double getLam5(int m) {
    double l;
    fgetlam5_(&m, &l);
    return l;
  }

  double getXmin(int m) {
    double xmin;
    fgetxmin_(&m, &xmin);
    return xmin;
  }

  double getXmax(int m) {
    double xmax;
    fgetxmax_(&m, &xmax);
    return xmax;
  }

  double getQ2min(int m) {
    double q2min;
    fgetq2min_(&m, &q2min);
    return q2min;
  }

  double getQ2max(int m) {
    double q2max;
    fgetq2max_(&m, &q2max);
    return q2max;
  }

  void extrapolate(bool extrapolate) { 
    if (extrapolate) {
      fextrapolateon_();
    } else {
      fextrapolateoff_();
    }
  }

  void setVerbosity(Verbosity noiselevel) { 
    switch (noiselevel) {
    case SILENT:
      fsilent_();
      break;
    case LOWKEY:
      flowkey_();
      break;
    default:
      fdefaultverb_();
    }
  }

  void setPDFPath(const string& path) { 
    char cpath[1000];
    strcpy(cpath, path.c_str());
    fsetpdfpath_(cpath, path.length());
  }


  // Functions below are the same as above but with M appended to their name.
  // They are for multiple set use in LHAPDF.  They also take an extra paramater
  // which is a label for their memory location 


  vector<double> xfxM(int nset, double x, double Q) {  
    vector<double> r(13);
    fevolvepdfm_(&nset, &x, &Q, &r[0]);
    return r;
  }

  double xfxM(int nset, double x, double Q, int fl) {  
    vector<double> r(13);
    fevolvepdfm_(&nset, &x, &Q, &r[0]);
    return r[fl+6];
  }

  vector<double> xfxpM(int nset, double x, double Q, double P2, int ip) {  
    vector<double> r(13);
    fevolvepdfpm_(&nset, &x, &Q, &P2,&ip, &r[0]);
    return r;
  }

  double xfxpM(int nset, double x, double Q, double P2, int ip, int fl) {  
    vector<double> r(13);
    fevolvepdfpm_(&nset, &x, &Q, &P2, &ip, &r[0]);
    return r[fl+6];
  }

  vector<double> xfxaM(int nset, double x, double Q, double a) {  
    vector<double> r(13);
    fevolvepdfam_(&nset, &x, &Q, &a, &r[0]);
    return r;
  }

  double xfxaM(int nset, double x, double Q, double a, int fl) {  
    vector<double> r(13);
    fevolvepdfam_(&nset, &x, &Q, &a, &r[0]);
    return r[fl+6];
  }

  vector<double> xfxphotonM(int nset, double x, double Q) {  
    vector<double> r(13);
    double mphoton;
    fevolvepdfphotonm_(&nset, &x, &Q, &r[0], &mphoton );
    r.push_back(mphoton);
    return r;
  }

  double xfxphotonM(int nset, double x, double Q, int fl) {  
    vector<double> r(13);
    double mphoton;
    fevolvepdfphotonm_(&nset, &x, &Q, &r[0], &mphoton);
    if ( fl == 7 ) return mphoton;
    return r[fl+6];
  }

  int numberPDFM(int nset) {
    int N; 
    fnumberpdfm_(&nset, &N); 
    return N;
  }

  double alphasPDFM(int nset, double Q) {
    double a;
    falphaspdfm_(&nset, &Q, &a);
    return a;
  }

  int getOrderPDFM(int nset) {
    int N;
    fgetorderpdfm_(&nset, &N);
    return N;
  }

  int getOrderAlphaSM(int nset) {
    int N;
    fgetorderasm_(&nset, &N);
    return N;
  }

  double getQMassM(int nset, int f) {
    double m;
    fgetqmassm_(&nset, &f, &m);
    return m;
  }

  double getThresholdM(int nset, int f) {
    double m;
    fgetthresholdm_(&nset, &f, &m);
    return m;
  }

  int getNfM(int nset) {
    int N; 
    fgetnfm_(&nset, &N); 
    return N;
  }

  double getLam4M(int nset, int m) {
    double l;
    fgetlam4m_(&nset, &m, &l);
    return l;
  }

  double getLam5M(int nset, int m) {
    double l;
    fgetlam5m_(&nset, &m, &l);
    return l;
  }

  double getXminM(int nset,int m) {
    double xmin;
    fgetxminm_(&nset, &m, &xmin);
    return xmin;
  }

  double getXmaxM(int nset,int m) {
    double xmax;
    fgetxmaxm_(&nset, &m, &xmax);
    return xmax;
  }

  double getQ2minM(int nset,int m) {
    double q2min;
    fgetq2minm_(&nset, &m, &q2min);
    return q2min;
  }

  double getQ2maxM(int nset,int m) {
    double q2max;
    fgetq2maxm_(&nset, &m, &q2max);
    return q2max;
  }

}
